@@ -35,9 +35,9 @@ make.par.list <- function(nams, x, n, m, def, recycle = sum(unnamed) > 0) {
 
 plot.zoo <- function(x, y = NULL, screens, plot.type, panel = lines, 
   xlab = "Index", ylab = NULL, main = NULL, xlim = NULL, ylim = NULL,
-  xy.labels = FALSE, xy.lines = NULL,
-  oma = c(6, 0, 5, 0), mar = c(0, 5.1, 0, 2.1), 
-  col = 1, lty = 1, lwd = 1, pch = 1, type = "l", 
+  xy.labels = FALSE, xy.lines = NULL, yax.flip = FALSE,
+  oma = c(6, 0, 5, 0), mar = c(0, 5.1, 0, if(yax.flip) 5.1 else 2.1), 
+  col = 1, lty = 1, lwd = 1, pch = 1, type = "l", log = "",
   nc, widths = 1, heights = 1, ...)
 {
   ## if y supplied: scatter plot y ~ x
@@ -57,7 +57,7 @@ plot.zoo <- function(x, y = NULL, screens, plot.type, panel = lines,
     ptype <- if(do.lab) "n" else if(missing(type)) "p" else type
 
     plot.default(xy, type = ptype,col = col, pch = pch, main = main,
-      xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim, ...)
+      xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim, log = log, ...)
     if(do.lab) text(xy, col = col,
       labels = if(!is.logical(xy.labels)) xy.labels else index2char(index(xyzoo)), ...)
     if(xy.lines) lines(xy, col = col, lty = lty, lwd = lwd, type = if(do.lab) "c" else "l", ...)
@@ -135,22 +135,23 @@ plot.zoo <- function(x, y = NULL, screens, plot.type, panel = lines,
 	ranges <- tapply(1:ncol(x), screens, f)
     for(j in seq_along(levels(screens))) {
       panel.number <- j
+      y.side <- if (j %% 2 || !yax.flip) 2 else 4
       range. <- rep(ranges[[j]], length.out = length(time(x)))
       if(j%%nr==0 || j == length(levels(screens))) {
-			args <- list(x.index, range., xlab = "", ylab = ylab[j], 
-				xlim = xlim, ylim = ylim[[j]], ...)
+			args <- list(x.index, range., xlab = "", ylab = "", yaxt = "n",
+				xlim = xlim, ylim = ylim[[j]], log = log, ...)
 			args$type <- "n"
 			do.call("plot", args)
 			mtext(xlab, side = 1, line = 3)
       } else {      
-			# args <- list(x.index, range., axes = FALSE, xlab = "", 
-			args <- list(x.index, range., xaxt = "n", xlab = "", 
-				ylab = ylab[j], xlim = xlim, ylim = ylim[[j]], ...)
+			args <- list(x.index, range., xaxt = "n", yaxt = "n", xlab = "", 
+				ylab = "", xlim = xlim, ylim = ylim[[j]], log = log, ...)
 			args$type <- "n"
 			do.call("plot", args)
-			box()
-			# axis(2, xpd = NA)
+			if ("bty" %in% names(args) && args$bty == "n") {} else box()
       }
+      do.call("axis", c(list(side = y.side, xpd = NA), dots))
+      mtext(ylab[j], y.side, line = 3)
 
       for(i in which(screens == levels(screens)[j]))
         panel(x.index, x[, i], col = col[[i]], pch = pch[[i]], lty = lty[i], lwd = lwd[i], type = type[[i]], ...)
@@ -171,17 +172,20 @@ plot.zoo <- function(x, y = NULL, screens, plot.type, panel = lines,
     dummy <- rep(range(x, na.rm = TRUE), 
 	length.out = length(index(x)))
 	    
-    args <- list(x.index, dummy, xlab = xlab, ylab = ylab[1], ylim = ylim, xlim = xlim, ...)
+    args <- list(x.index, dummy, xlab = xlab, ylab = ylab[1], ylim = ylim, xlim = xlim, log = log, ...)
     args$type <- "n"
     do.call("plot", args)
-    box()
+	if ("bty" %in% names(args) && args$bty == "n") {} else box()
     y <- as.matrix(x)
     for(i in 1:nser) {
       panel(x.index, y[, i], col = col[[i]], pch = pch[[i]], lty = lty[i], 
         lwd = lwd[i], type = type[[i]], ...)
     }
   }
-  title(main, outer = main.outer)
+  dots <- list(...)
+  title.args <- c(list(main = main, outer = main.outer),
+    dots[grep("[.]main$", names(dots))])
+  do.call("title", title.args)
   return(invisible(x))
 }
 
